<?php

/*
Plugin Name: iVolunteer
Plugin URI: http://wordpress.org/extend/plugins/iVolunteer/
Description: Retrieves local community volunteer opportunities and displays them in the sidebar. After activating the Plugin, go to Widgets to add iVolunteer to your sidebar.
Version: 1.0.1
Author: Neil Simon
Author URI: http://actionfeed.org/
*/


/*
 Copyright 2009 iVolunteer.

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin St, 5th Floor, Boston, MA 02110 USA
*/


// Constants
define ('IVOLUNTEER_PLUGIN_VERSION',    '1.0.1');
define ('IVOLUNTEER_SETTINGS_OPTIONS',  'iVolunteer_settingsOptions');
define ('IVOLUNTEER_API_URL',           'http://www.actionfeed.org/api/return_events/');
define ('IVOLUNTEER_URL',               'http://www.actionfeed.org/');


// Global data -- used exclusively by iVolunteeer_xml*(), and iVolunteer_loadEventArray().
// These globals are required, due to the way xml_*() callbacks are implemented by PHP.
$gInsideEVENT = FALSE;
$gTag         = '';
$gTitle       = '';
$gTime        = '';
$gLocation    = '';
$gPhone       = '';
$gUrl         = '';
$gIndex       = 0;
$gEventList   = array (array ('title' => '', 'time' => '', 'location' => '', 'phone' => '', 'url' => ''));


function iVolunteer_displayEventArrayInSidebar ()
    {
    global $gEventList;

    iVolunteer_loadEventArray ();

    // Initialize output buffer (returned by this function)
    $buf = '';

    // Open list
    $buf .= '<ul>';

    // Add events
    for ($i = 0; $gEventList [$i] ['title'] != ''; $i++)
        {
        // Get next event 
        $eventTitle    = $gEventList [$i] ['title'];
        $eventTime     = $gEventList [$i] ['time'];
        $eventLocation = $gEventList [$i] ['location'];
        $eventPhone    = $gEventList [$i] ['phone'];
        $eventUrl      = $gEventList [$i] ['url'];

        // Display list item
        $buf .= sprintf ("<li>%s<br />%s<br />%s<br /><b>%s</b><br /><a href=\"%s\" title=\"%s\">%s</a></li><br />",
                          $eventTitle, $eventTime, $eventLocation, $eventPhone, $eventUrl, $eventTitle, $eventUrl);
        }

    // Close list
    $buf .= '</ul>';

    // $buf will be displayed in the sidebar
    return ($buf);
    }


function iVolunteer_loadEventArray ()
    {
    // Get the iVolunteer local events list -- returned as XML
    if (($return_events_response = iVolunteer_cURL ()) == FALSE)
        {
        // This can happen when the site is found, but the API is down
        // -- just exit
        }

    elseif (strpos ($return_events_response, "Not Found"))
        {
        // This can happen when the site is NOT found
        // -- just exit
        }

    else
        {
        global $gIndex, $gInsideEVENT, $gEventList;

        // Initialize XML-related globals
        $gIndex       = 0;
        $gInsideEVENT = FALSE;

        // Initialize cache settings to null -- the XML parsing will load this up
        $gEventList = array (array ('title' => '', 'time' => '', 'location' => '', 'phone' => '', 'url' => ''));

        // Example xml returned:
        //
        // <xml version="1.0" encoding="iso-8859-1">
        // <events>
        //   <event>
        //     <title>Sister Carmen Food Pantry Monthly Sort</title>
        //     <time>Sat Feb 21 2009 2:00pm-4:00pm MST</time>
        //     <location>701 W. Baseline Road, Lafayette, CO 80026</location>
        //     <phone>303-665-4342</phone>
        //     <url>http://www.sistercarmen.org</url>
        //   </event>
        //   <event>
        //     <title>Earth Day Tree Planting Event</title>
        //     <time>Wed Apr 22 2009 1:00pm-5:00pm MST</time>
        //     <location>26th Avenue and York Street, Denver, CO 80211</location>
        //     <phone>303-555-1212</phone>
        //     <url>http://www.denvergov.org/parks_recreation</url>
        //   </event>
        // </events>

        // Prepare to parse
        $xmlParser = xml_parser_create ();

        // Define XML callback -- called via xml_parse()
        xml_set_element_handler ($xmlParser, 'iVolunteer_xmlStartElement', 'iVolunteer_xmlEndElement');

        // Define XML callback -- called via xml_parse()
        xml_set_character_data_handler ($xmlParser, 'iVolunteer_xmlData');

        // Parse the XML -- all data will be stored into gEventList
        xml_parse ($xmlParser, $return_events_response, TRUE);

        // Set last item url in gEventList to null
        $gEventList [$gIndex] ['title'] = '';

        // Release parser resources
        xml_parser_free ($xmlParser);
        }
    }


function iVolunteer_cURL ()
    {
    // TESTING
    return (file_get_contents ('wp-content/plugins/ivolunteer/mockup.xml'));

    $curlUrl = IVOLUNTEER_API_URL . $_SERVER ['REMOTE_ADDR'];

    // Create a new cURL resource
    $ch = curl_init ();

    curl_setopt ($ch, CURLOPT_POST,           1);
    curl_setopt ($ch, CURLOPT_POSTFIELDS,     '');
    curl_setopt ($ch, CURLOPT_URL,            $curlUrl);
    curl_setopt ($ch, CURLOPT_RETURNTRANSFER, TRUE);

    // Prepend @ to this call to suppress (in some environments):
    // "Warning curl_setopt(): CURLOPT_FOLLOWLOCATION cannot be activated
    //  when in safe_mode or an open_basedir is set in"
    // This was reported by one user in WordPress forum, topic: 235020
    // User mentioned that the code works fine -- even though this warning is displayed
    @curl_setopt ($ch, CURLOPT_FOLLOWLOCATION, TRUE);

    // Exec the URL
    $response = curl_exec ($ch);

    // Close cURL resource, and free up system resources
    curl_close ($ch);

    return ($response);
    }


function iVolunteer_xmlStartElement ($parser, $name, $attrs)
    {
    global $gInsideEVENT, $gTag;

    // This function gets called each time <xxxx> (any start element tag) encountered

    // If inside a subtag of <event>...
    if ($gInsideEVENT)
        {
        // Possible values for gTag can be: TITLE, TIME, LOCATION, PHONE, URL
        // NOTE: PHP converts the tags to uppercase
        $gTag = $name;
        }

    // If not already within the <event> tag, check to see if this is a <event> tag
    elseif ($name == 'EVENT')
        {
        // Let iVolunteer_xmlData() know that we're inside the EVENT tag
        $gInsideEVENT = TRUE;
        }
    }


function iVolunteer_xmlEndElement ($parser, $name)
    {
    global $gInsideEVENT, $gTitle, $gTime, $gLocation, $gPhone, $gUrl, $gEventList, $gIndex;

    // This function gets called each time ANY </xxxx> is encountered

    // When xml_parser() detects </event>...
    if ($name == 'EVENT')
        {
        // Update global cache vars from global data fields
        $gEventList [$gIndex] ['title']    = $gTitle;
        $gEventList [$gIndex] ['time']     = $gTime;
        $gEventList [$gIndex] ['location'] = $gLocation;
        $gEventList [$gIndex] ['phone']    = $gPhone;
        $gEventList [$gIndex] ['url']      = $gUrl;

        // Clear global data fields
        $gTitle       = '';
        $gTime        = '';
        $gLocation    = '';
        $gPhone       = '';
        $gUrl         = '';
        $gInsideEVENT = FALSE;
        $gIndex++;
        }
    }


function iVolunteer_xmlData ($parser, $data)
    {
    global $gInsideEVENT, $gTag, $gTitle, $gTime, $gLocation, $gPhone, $gUrl;

    // This function gets called iteratively, one time per each subtag with <event>

    // Process the subtag -- save the set of values to globals
    if ($gInsideEVENT)
        {
        switch ($gTag)
            {
            case 'TITLE':
                $gTitle .= $data;
                break;

            case 'TIME':
                $gTime .= $data;
                break;

            case 'LOCATION':
                $gLocation .= $data;
                break;

            case 'PHONE':
                $gPhone .= $data;
                break;

            case 'URL':
                $gUrl .= $data;
                break;
            }
        }
    }


function iVolunteer_buildSidebarList ()
    {
    // Build sidebar table and display it
    echo (iVolunteer_displayEventArrayInSidebar ());
    }


function iVolunteer_initWidget ()
    {
    // MUST be able to register the widget... else exit
    if (function_exists ('register_sidebar_widget'))
        {
        // Declare function -- called from Wordpress -- during page-loads
        function iVolunteer_widget ($args)
            {
            // Load existing options from wp database
            $iVolunteer_settingsOptions = get_option (IVOLUNTEER_SETTINGS_OPTIONS);

            // Accept param array passed-in from WP (e.g. $before_widget, $before_title, etc.), theme CSS styles
            extract ($args);

            // Display sidebar title above the about-to-be-rendered iVolunteer events table
            echo $before_widget . $before_title .
                 '<a href="' . IVOLUNTEER_URL . '">' . $iVolunteer_settingsOptions ['sidebarTitle'] . '</a>' .
                 $after_title;

            // Open a plugin-version-tracking DIV tag
            printf ("<div id=\"%s\">", IVOLUNTEER_PLUGIN_VERSION);

            // Dynamically build the table and display it
            iVolunteer_buildSidebarList ();

            // Close the plugin-version-tracking DIV tag
            printf ("</div>");

            echo $after_widget;
            }

        // Register the widget function to be called from Wordpress on each page-load
        register_sidebar_widget ('iVolunteer', 'iVolunteer_widget');
        }
    }


function iVolunteer_createOptions ()
    {
    // Create the initialSettingsOptions array of keys/values
    $iVolunteer_initialSettingsOptions = array ('sidebarTitle' => 'iVolunteer');

    // Store the initial options to the wp database
    add_option (IVOLUNTEER_SETTINGS_OPTIONS, $iVolunteer_initialSettingsOptions);
    }


function iVolunteer_deleteOptions ()
    {
    // Remove the iVolunteer_settingsOptions array from the wp database
    delete_option (IVOLUNTEER_SETTINGS_OPTIONS);
    }


function iVolunteer_updateSettingsPage ()
    {
    // Load existing options from wp database
    $iVolunteer_settingsOptions = get_option (IVOLUNTEER_SETTINGS_OPTIONS);

    // Localize all displayed strings
    $iVolunteer_enterOptionsStr = __('Please enter the iVolunteer Plugin options:',   'iVolunteer');
    $iVolunteer_sidebarTitleStr = __('Sidebar title to display:',                     'iVolunteer');
    $iVolunteer_saveStr         = __('Save',                                          'iVolunteer');
    $iVolunteer_optionsSavedStr = __('iVolunteer Plugin options saved successfully.', 'iVolunteer');

    // If ALL data fields contain values...
    if (isset ($_POST ['sidebarTitle']))
        {
        // Copy the fields to the persistent wp options array
        $iVolunteer_settingsOptions ['sidebarTitle'] = $_POST ['sidebarTitle'];

        // Store changed options back to wp database
        update_option (IVOLUNTEER_SETTINGS_OPTIONS, $iVolunteer_settingsOptions);

        // Display update message to user
        echo '<div id="message" class="updated fade"><p>' . $iVolunteer_optionsSavedStr . '</p></div>';
        }

    // Display form to the user

    echo
     '<div class="wrap">

      <h3>&nbsp;' . $iVolunteer_enterOptionsStr . '</h3>

      <form action="" method="post">

      <table border="0" cellpadding="10">

      <tr>
      <td>' . $iVolunteer_sidebarTitleStr . ' &nbsp;</td>
      <td><input type="text" name="sidebarTitle" value="' . $iVolunteer_settingsOptions ['sidebarTitle'] . '" size="40" /></td>
      </tr>

      </table>

      <br />

      <p>&nbsp;<input type="submit" value="' . $iVolunteer_saveStr . '" /></p>

      </form>

      </div>';
    }


function iVolunteer_addSubmenu ()
    {
    // Define the options for the submenu page
    add_submenu_page ('options-general.php',             // Parent page
                      'iVolunteer page',                 // Page title, shown in titlebar
                      'iVolunteer',                      // Menu title
                      10,                                // Access level all
                      __FILE__,                          // This file displays the options page
                      'iVolunteer_updateSettingsPage');  // Function that displays options page
    }


// Initialize for localized strings
//
// If (the WordPress config file "wp-config.php" has a language defined, e.g. ('WPLANG', 'it_IT'))
//     {
//     WordPress will look in the plugin directory for a corresponding localized strings file and load it
//     (e.g. "ivolunteer-it_IT.mo")
//     }
load_plugin_textdomain ('iVolunteer', 'wp-content/plugins/ivolunteer');


// Runs only once at activation time
register_activation_hook (__FILE__, 'iVolunteer_createOptions');


// Runs only once at deactivation time
register_deactivation_hook (__FILE__, 'iVolunteer_deleteOptions');


// Load the widget, show it in the widget control in the admin section
add_action ('plugins_loaded', 'iVolunteer_initWidget');


// Add the iVolunteer submenu to the Settings page
add_action ('admin_menu', 'iVolunteer_addSubmenu');

?>
